/*
 * Isomorphic SmartClient web presentation layer
 * Copyright 2000 and beyond Isomorphic Software, Inc.
 *
 * OWNERSHIP NOTICE
 * Isomorphic Software owns and reserves all rights not expressly granted in this source code,
 * including all intellectual property rights to the structure, sequence, and format of this code
 * and to all designs, interfaces, algorithms, schema, protocols, and inventions expressed herein.
 *
 *  CONFIDENTIALITY NOTICE
 *  The contents of this file are confidential and protected by non-disclosure agreement:
 *    - You may not expose this file to any person who is not bound by the same obligations.
 *    - You may not expose or send this file unencrypted on a public network.
 *
 *  If you have any questions, please email <sourcecode@isomorphic.com>.
 *
 *  This entire comment must accompany any portion of Isomorphic Software source code that is
 *  copied or moved from this file.
 */

package com.smartgwtee.client.messaging;

public class Messaging {

    /**
     * Subscribes the client to the messaging channel identified by channel. When the server sends a message on this
     * channel, the callback function or script will be called on the client with a single data parameter.
     *
     * @param channel  the channel
     * @param callback callback
     */
    public static native void subscribe(String channel, MessagingCallback callback) /*-{
        $wnd.isc.Messaging.subscribe(channel, function(data) {
            callback.@com.smartgwtee.client.messaging.MessagingCallback::execute(Ljava/lang/Object;)(data);
        });
    }-*/;

    /**
     * Unsubscribes the client from the messaging channel identified by channel.
     *
     * @param channel the channel to unsubscribe
     */
    public static native void unsubscribe(String channel) /*-{
            $wnd.isc.Messaging.unsubscribe(channel);
    }-*/;

    /**
     * Returns an array of identifiers for all currently subscribed channels on this client.
     *
     * @return the subscribed channels
     */
    public static native String[] getSubscribedChannels() /*-{
        var channelsJS = $wnd.isc.Messaging.getSubscribedChannels();
        return @com.smartgwt.client.util.JSOHelper::convertToJavaStringArray(Lcom/google/gwt/core/client/JavaScriptObject;)(channelsJS);
    }-*/;

    /**
     * Send any data (primitive type, or compound data structure) to a channel or list of channels. channels may be a
     * single channel identifier (string) or an array of channel identifiers. data will be passed to the callback
     * specified in any subscribe() call for a matching channel.
     *
     * @param channel the channel to send message to
     * @param data    the message payload
     */
    public static native void send(String channel, Object data) /*-{
            $wnd.isc.Messaging.send(channel, data);
    }-*/;


}

