/*
 * Isomorphic SmartGWT web presentation layer
 * Copyright 2000 and beyond Isomorphic Software, Inc.
 *
 * OWNERSHIP NOTICE
 * Isomorphic Software owns and reserves all rights not expressly granted in this source code,
 * including all intellectual property rights to the structure, sequence, and format of this code
 * and to all designs, interfaces, algorithms, schema, protocols, and inventions expressed herein.
 *
 *  If you have any questions, please email <sourcecode@isomorphic.com>.
 *
 *  This entire comment must accompany any portion of Isomorphic Software source code that is
 *  copied or moved from this file.
 */

package com.smartgwt.sample.showcase.client.data;

import com.smartgwt.sample.showcase.client.SmartGWTCommand;
import com.smartgwt.sample.showcase.client.dataintegration.java.crud.*;
import com.smartgwt.sample.showcase.client.dataintegration.java.form.FormValidationSample;
import com.smartgwt.sample.showcase.client.dataintegration.java.grid.*;
import com.smartgwt.sample.showcase.client.dataintegration.java.transactions.QueuedMasterDetailAddSample;
import com.smartgwt.sample.showcase.client.dataintegration.java.transactions.SimpleQueuingSample;
import com.smartgwt.sample.showcase.client.dataintegration.java.tree.TreeBindingSQLSample;
import com.smartgwt.sample.showcase.client.dataintegration.java.tree.TreeBindingServletSample;
import com.smartgwt.sample.showcase.client.dataintegration.java.tree.TreeReparentSample;
import com.smartgwt.sample.showcase.client.dataintegration.java.datasource.SimpleCustomDataSourceSample;
import com.smartgwt.sample.showcase.client.dataintegration.java.others.BatchUploaderSample;
import com.smartgwt.sample.showcase.client.doc.DataSourcesPanel;
import com.smartgwt.sample.showcase.client.doc.GettingStartedPanel;
import com.smartgwt.sample.showcase.client.doc.IntegrationOptionsPanel;
import com.smartgwt.sample.showcase.client.doc.IntroductionPanel;
import com.smartgwt.sample.showcase.client.tools.*;
import com.smartgwt.sample.showcase.client.webservice.RssSample;
import com.smartgwt.sample.showcase.client.webservice.WsdlDataBindingSample;
import com.smartgwt.sample.showcase.client.webservice.WsdlOperationSample;


public class ShowcaseData {

    private String idSuffix;

    public ShowcaseData(String idSuffix) {
        this.idSuffix = idSuffix;
    }

    private ExplorerTreeNode[] data;

    private ExplorerTreeNode[] getData() {
        if (data == null) {
            data = new ExplorerTreeNode[]{

                    new ExplorerTreeNode("SmartGWT EE", "doc-category", "root", "silk/book.png", null, true, idSuffix),
                    new ExplorerTreeNode("Introduction", "doc-introduction", "doc-category", "silk/book_open.png", new IntroductionPanel.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Server Integration", "doc-server-integration", "doc-category", "silk/computer_go.png", new DataSourcesPanel.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Integration Options", "doc-options", "doc-category", "silk/cog_go.png", new IntegrationOptionsPanel.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Getting Started", "doc-getting-started", "doc-category", "silk/help.png", new GettingStartedPanel.Factory(), true, idSuffix),

                    new ExplorerTreeNode("New Samples", "new-category", "root", "iconexperience/objects_exchange.png", null, true, idSuffix),
                    new ExplorerTreeNode("Basic Connector (SQL)", "grid-basic-connector-new", "new-category", "silk/database_gear.png", new BasicConnectorSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Server Advanced Filter (SQL)", "filterbuilder-sql-new", "new-category", "iconexperience/funnel.png", new ServerAdvancedFilteringSQLSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Server Advanced Filter (Hibernate)", "filterbuilder-hibernate-new", "new-category", "iconexperience/funnel.png", new ServerAdvancedFilteringHibernateSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Large Value Map (SQL)", "large-valuemap-sql-new", "new-category", "silk/table_relationship.png", new LargeValueMapSQLSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Master-Detail Batch Load &amp; Save", "mater-detail-batch-new", "new-category", "silk/table_multiple.png", new MasterDetailHibernateSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Flattened Data Model", "flattened-datamodel-new", "new-category", "iconexperience/branch.png", new FlattenedDataModelSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("User Specific Data (SQL)", "user-specific-data-new", "new-category", "silk/user_orange.png", new UserSpecificDataSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Dynamic Reporting (SQL)", "dynamic-reporting-new", "new-category", "silk/table_multiple.png", new DynamicReportingSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Excel Export", "excel-export-new", "new-category", "silk/page_white_excel.png", new ExcelExportSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Batch File Uploader", "batch-uploader-new", "new-category", "silk/page_white_excel.png", new BatchUploaderSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Simple Queuing", "transactions-queuing-new", "new-category", "gears.png", new SimpleQueuingSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Custom DataSource", "simple-custom-ds-new", "new-category", "silk/table_row_insert.png", new SimpleCustomDataSourceSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("DataBase Browser", "tools-db-browser-wizard-new", "new-category", "silk/database_gear.png", new DatabaseWizardBrowserStubPanel.Factory(), true, idSuffix),
                    new ExplorerTreeNode("DataSource Editor", "tools-db-editor-wizard-new", "new-category", "silk/database_gear.png", new DataSourceWizardEditorStubPanel.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Hibernate Wizard", "tools-hibernate-wizard-new", "new-category", "iconexperience/coffeebean.png", new HibernateWizardStubPanel.Factory(), true, idSuffix),


                    new ExplorerTreeNode("Java Data Integration", "data-integration-java", "root", "silk/database_connect.png", null, true, idSuffix),


                    new ExplorerTreeNode("CRUD", "data-integration-crud", "data-integration-java", "silk/database_table.png", null, true, idSuffix),
                    new ExplorerTreeNode("Relational Connector (SQL)", "data-integration-crud-sql", "data-integration-crud", "silk/database_gear.png", new SqlCrudSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Direct Method Invocation (DMI)", "data-integration-crud-dmi", "data-integration-crud", "silk/database_edit.png", new DmiCrudSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Hibernate (Beanless)", "data-integration-crud-hibernate-prototyping", "data-integration-crud", "silk/database_save.png", new HibernatePrototypeCrudSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Spring-Hibernate (Production)", "data-integration-crud-hibernate-production", "data-integration-crud", "silk/database_save.png", new HibernateProductionCrudSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Servlet CRUD ", "data-integration-crud-servlet", "data-integration-crud", "silk/server_connect.png", new ServletControllerCrudSample.Factory(), true, idSuffix),

                    new ExplorerTreeNode("Forms", "data-integration-form", "data-integration-java", "silk/application_form.png", null, true, idSuffix),
                    new ExplorerTreeNode("Form Client &amp; Server Validation", "data-integration-form-validation", "data-integration-form", "silk/table_row_delete.png", new FormValidationSample.Factory(), true, idSuffix),

                    new ExplorerTreeNode("Transactions", "transactions", "data-integration-java", "silk/server_lightning.png", null, true, idSuffix),
                    new ExplorerTreeNode("Simple Queuing", "transactions-queuing", "transactions", "gears.png", new SimpleQueuingSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Queued Master / Detail Add", "transactions-queued-md", "transactions", "silk/table_row_insert.png", new QueuedMasterDetailAddSample.Factory(), true, idSuffix),

                    new ExplorerTreeNode("Grid", "data-integration-grid", "data-integration-java", "silk/application_view_detail.png", null, true, idSuffix),
                    new ExplorerTreeNode("JavaBeans (DMI)", "data-integration-grid-dmi", "data-integration-grid", "iconexperience/coffeebean.png", new JavaBeansGridSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Basic Connector (SQL)", "data-integration-grid-basic-connector", "data-integration-grid", "silk/database_gear.png", new BasicConnectorSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Editable Live Grid (SQL)", "data-integration-databinding-livegrid", "data-integration-grid", "silk/application_put.png", new LiveGridFetchSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Server Advanced Filter (SQL)", "data-integration-filterbuilder-sql", "data-integration-grid", "iconexperience/funnel.png", new ServerAdvancedFilteringSQLSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Server Advanced Filter (Hibernate)", "data-integration-filterbuilder-hibernate", "data-integration-grid", "iconexperience/funnel.png", new ServerAdvancedFilteringHibernateSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Large Value Map (SQL)", "data-integration-large-valuemap-sql", "data-integration-grid", "silk/table_relationship.png", new LargeValueMapSQLSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Master-Detail Batch Load &amp; Save", "data-integration-mater-detail-batch", "data-integration-grid", "silk/table_multiple.png", new MasterDetailHibernateSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Flattened Data Model", "data-integration-flattened-datamodel", "data-integration-grid", "iconexperience/branch.png", new FlattenedDataModelSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("User Specific Data (SQL)", "data-integration-user-specific-data", "data-integration-grid", "silk/user_orange.png", new UserSpecificDataSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Dynamic Reporting (SQL)", "data-integration-databinding-dynamic-reporting", "data-integration-grid", "silk/table_multiple.png", new DynamicReportingSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Excel Export", "excel-export", "data-integration-grid", "silk/page_white_excel.png", new ExcelExportSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Batch File Uploader", "batch-uploader", "data-integration-grid", "silk/page_white_excel.png", new BatchUploaderSample.Factory(), true, idSuffix),

                    new ExplorerTreeNode("Tree", "data-integration-tree", "data-integration-java", "silk/chart_organisation.png", null, true, idSuffix),
                    new ExplorerTreeNode("Load on Demand (SQL)", "data-integration-tree-sql", "data-integration-tree", null, new TreeBindingSQLSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Load Beans on Demand (Servlet)", "data-integration-tree-servlet", "data-integration-tree", null, new TreeBindingServletSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Tree Reparent", "data-integration-databinding-tree-reparent", "data-integration-tree", null, new TreeReparentSample.Factory(), true, idSuffix),

                    new ExplorerTreeNode("Custom DataSources", "custom-ds", "data-integration-java", "silk/server_lightning.png", null, true, idSuffix),
                    new ExplorerTreeNode("Simple", "simple-custom-ds", "custom-ds", "silk/table_row_insert.png", new SimpleCustomDataSourceSample.Factory(), true, idSuffix),

                    new ExplorerTreeNode("WebServices (WSDL) and RSS", "data-integration-ws-rss", "root", "silk/cog_go.png", null, true, idSuffix),
                    new ExplorerTreeNode("WSDL operation (generic)", "data-integration-server-wsdl-generic", "data-integration-ws-rss", "silk/cog_go.png", new WsdlOperationSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("WSDL databinding (Google Search)", "data-integration-server-wsdl-fetch", "data-integration-ws-rss",  "silk/cog.png", new WsdlDataBindingSample.Factory(), true, idSuffix),
                    new ExplorerTreeNode("RSS ListGrid binding", "data-integration-server-rss", "data-integration-ws-rss", "silk/feed.png", new RssSample.Factory(), true, idSuffix),

                    new ExplorerTreeNode("Tools", "tools-category", "root", "silk/wrench_orange.png", null, true, idSuffix),

                    new ExplorerTreeNode("DataSource Wizard", "tools-ds-wizard", "tools-category", "silk/database_lightning.png", new DataSourceGeneratorPanel.Factory(), true, idSuffix),
                    //new ExplorerTreeNode("DataSource Wizard", "tools-ds-wizard", "tools-category", "silk/database_lightning.png", new DataSourceGeneratorStubPanel.Factory(), true, idSuffix),

                    new ExplorerTreeNode("DataBase Browser", "tools-db-browser-wizard", "tools-category", "silk/database_gear.png", new DatabaseWizardBrowserStubPanel.Factory(), true, idSuffix),
                    new ExplorerTreeNode("DataSource Editor", "tools-db-editor-wizard", "tools-category", "silk/database_gear.png", new DataSourceWizardEditorStubPanel.Factory(), true, idSuffix),
                    new ExplorerTreeNode("Hibernate Wizard", "tools-hibernate-wizard", "tools-category", "iconexperience/coffeebean.png", new HibernateWizardStubPanel.Factory(), true, idSuffix),

                    //new ExplorerTreeNode("DataSource Admin Console", "tools-admin-console", "tools-category", "silk/server_database.png", new DataSourceConsoleStubPanel.Factory(), true, idSuffix),
                    new CommandTreeNode("DataSource Admin Console", "tools-admin-console", "tools-category", "silk/server_database.png", new DataSourceConsoleCommand(), true, idSuffix) {
                        {
                            setDescription("SmartGWT DataSource Administrator console.");
                        }
                    },

                    //new ExplorerTreeNode("SmartClient Visual Builder", "tools-visualbuilder", "tools-category", "silk/palette.png", new VisualBuilderStubPanel.Factory(), true, idSuffix),
                    new CommandTreeNode("SmartClient Visual Builder", "tools-visualbuilder", "tools-category", "silk/palette.png", new VisualBuilderCommand(), true, idSuffix) {
                        {
                            setDescription("SmartClient's powerful WYSWIG tool with ability to connect DataSource's to DataBound components.");
                        }
                    },

                    new CommandTreeNode("Developer Console", "tools-developer-console", "tools-category", "silk/bug.png", new DebugConsoleCommand(), true, idSuffix) {
                        {
                            setDescription("SmartGWT Developer console for troubleshooting, viewing client & server logs, and more.. ");
                        }
                    },

                    new CommandTreeNode("SmartGWT Showcase", "smartgwt-category", "root", "silk/house.png", new SmartGWTCommand(), true, idSuffix),
            };
        }
        return data;
    }

    public static ExplorerTreeNode[] getData(String idSuffix) {
        return new ShowcaseData(idSuffix).getData();
    }
}
