/*
 * Isomorphic SmartGWT web presentation layer
 * Copyright 2000 and beyond Isomorphic Software, Inc.
 *
 * OWNERSHIP NOTICE
 * Isomorphic Software owns and reserves all rights not expressly granted in this source code,
 * including all intellectual property rights to the structure, sequence, and format of this code
 * and to all designs, interfaces, algorithms, schema, protocols, and inventions expressed herein.
 *
 *  If you have any questions, please email <sourcecode@isomorphic.com>.
 *
 *  This entire comment must accompany any portion of Isomorphic Software source code that is
 *  copied or moved from this file.
 */

package com.smartgwt.sample.showcase.client.tools;

import com.smartgwt.client.widgets.*;
import com.smartgwt.client.widgets.layout.VLayout;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.sample.showcase.client.PanelFactory;
import com.smartgwt.sample.showcase.client.ShowcasePanel;


public class DataSourceGeneratorPanel extends ShowcasePanel {
    
    private static final String DESCRIPTION = "<p>The SamrtGWT EE DataSource generator is the key starting point for " +
            "integrating with any pre-existing system: it generates DataSources from existing SQL tables or Java beans " +
            "(Hibernate or otherwise), and in the case of SQL or Hibernate the generated DataSources are immediately fully " +
            "functional for all CRUD operations, including advanced filtering.</p>" +
            "<p>To generate a datasource from an existing database table, click the <b>DataSource Generator</b> button and in " +
            "the 'List of table name' field enter <b>ANIMALS</b> or <b>TEAMMEMBERS</b>. Make sure you select the 'Overwite files if they exist' " +
            "checkbox as someone else might have generated the datasource prior to you. Now click the <b>OK</b> button and the datasource is generated " +
            " from the tabel definition</p>" +
            "<p>You can check out the datasource that you just generated by opening the <b>DataSource Admin Console</b> from the side nav</p>" +
            "<p>If you'd like to generate a datasoruce from an existing server side Javabean, repeat the above steps but instead of entering a table name" +
            " in the generator, enter the name of a server side bean. For example : <b>com.smartgwt.sample.showcase.server.SupplyItemBean</b></p>";



    public static class Factory implements PanelFactory {

        private String id;

        public Canvas create() {
            DataSourceGeneratorPanel panel = new DataSourceGeneratorPanel();
            id = panel.getID();
            return panel;
        }

        public String getID() {
            return id;
        }

        public String getDescription() {
            return DESCRIPTION;
        }
    }

    public String getIntro() {
        return DESCRIPTION;
    }

    protected boolean isTopIntro() {
        return true;
    }

    public Canvas getViewPanel() {

        IButton button = new IButton("DataSource Generator");
        button.setIcon("silk/database_lightning.png");
        button.setAutoFit(true);
        button.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                com.google.gwt.user.client.Window.open("./tools/dsGenerator.jsp", "ds_wizard_blank", null);
            }
        });
        VLayout layout = new VLayout(15);
        
        layout.setWidth100();
        layout.addMember(button);
        return layout;
    }
}